@@ -22,12 +22,9 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE. */
 
-#include <string.h>
-#include <stdlib.h>
-#include <sys/types.h>
-#include <errno.h>
+/* Handle the multiplexed channels, such as sessions, x11, agent connections */
 
-#include "options.h"
+#include "includes.h"
 #include "session.h"
 #include "packet.h"
 #include "ssh.h"
@@ -38,6 +35,7 @@
 #include "ssh.h"
 #include "x11fwd.h"
 #include "agentfwd.h"
+#include "localtcpfwd.h"
 
 static void send_msg_channel_open_failure(unsigned int remotechan, int reason,
 		const unsigned char *text, const unsigned char *lang);
@@ -52,7 +50,9 @@ static void send_msg_channel_data(struct Channel *channel, int isextended,
 static void send_msg_channel_eof(struct Channel *channel);
 static void send_msg_channel_close(struct Channel *channel);
 static void closechannel(struct Channel *channel);
+static void deletechannel(struct Channel *channel);
 static void send_exitsignalstatus(struct Channel *channel);
+static void checkinitdone(struct Channel *channel);
 
 /* initialise channels */
 void chaninitialise() {
@@ -65,9 +65,10 @@ void chaninitialise() {
 	chansessinitialise();
 }
 
+/* Clean up channels, freeing allocated memory */
 void chancleanup() {
 
-	int i;
+	unsigned int i;
 
 	TRACE(("enter chancleanup"));
 	for (i = 0; i < ses.chansize; i++) {
@@ -88,7 +89,7 @@ struct Channel* newchannel(unsigned int remotechan, unsigned char type,
 		unsigned int transwindow, unsigned int transmaxpacket) {
 
 	struct Channel * newchan;
-	int i, j;
+	unsigned int i, j;
 
 	TRACE(("enter newchannel"));
 	
@@ -133,6 +134,7 @@ struct Channel* newchannel(unsigned int remotechan, unsigned char type,
 	newchan->infd = -1;
 	newchan->outfd = -1;
 	newchan->errfd = -1;
+	newchan->initconn = 0;
 
 	newchan->writebuf = buf_new(RECV_MAXWINDOW);
 	newchan->recvwindow = RECV_MAXWINDOW;
@@ -145,6 +147,7 @@ struct Channel* newchannel(unsigned int remotechan, unsigned char type,
 	return newchan;
 }
 
+/* Get the channel structure corresponding to a channel number */
 static struct Channel* getchannel(unsigned int chan) {
 	if (chan >= ses.chansize || ses.channels[chan] == NULL) {
 		return NULL;
@@ -152,10 +155,11 @@ static struct Channel* getchannel(unsigned int chan) {
 	return ses.channels[chan];
 }
 
+/* Iterate through the channels, performing IO if available */
 void channelio(fd_set *readfd, fd_set *writefd) {
 
 	struct Channel *channel;
-	int i;
+	unsigned int i;
 
 	for (i = 0; i < ses.chansize; i++) {
 
@@ -177,7 +181,13 @@ void channelio(fd_set *readfd, fd_set *writefd) {
 		/* write to program/pipe stdin */
 		if (channel->infd != -1 && channel->recveof == 0 &&
 				FD_ISSET(channel->infd, writefd)) {
-			writechannel(channel);
+			if (channel->initconn) {
+				checkinitdone(channel);
+				continue; /* Important not to use the channel after
+							 checkinitdone(), as it may be NULL */
+			} else {
+				writechannel(channel);
+			}
 		}
 		
 		/* handle any listening sockets */
@@ -198,6 +208,31 @@ void channelio(fd_set *readfd, fd_set *writefd) {
 
 }
 
+/* Check whether a deferred (EINPROGRESS) connect() was successful, and
+ * if so, set up the channel properly. Otherwise, the channel is cleaned up, so
+ * it is important that the channel reference isn't used after a call to this
+ * function */
+static void checkinitdone(struct Channel *channel) {
+
+	int val;
+	socklen_t vallen = sizeof(val);
+
+	TRACE(("enter checkinitdone"));
+
+	if (getsockopt(channel->infd, SOL_SOCKET, SO_ERROR, &val, &vallen)
+			|| val != 0) {
+		close(channel->infd);
+		deletechannel(channel);
+		TRACE(("leave checkinitdone: fail"));
+	} else {
+		channel->outfd = channel->infd;
+		channel->initconn = 0;
+		TRACE(("leave checkinitdone: success"));
+	}
+}
+
+
+/* send the exit status or the signal causing termination for a session */
 static void send_exitsignalstatus(struct Channel *channel) {
 
 	struct ChanSess * chansess;
@@ -213,6 +248,7 @@ static void send_exitsignalstatus(struct Channel *channel) {
 }
 
 
+/* Send the close message and set the channel as closed */
 static void send_msg_channel_close(struct Channel *channel) {
 
 	TRACE(("enter send_msg_channel_close"));
@@ -248,7 +284,9 @@ static void send_msg_channel_eof(struct Channel *channel) {
 	TRACE(("leave send_msg_channel_eof"));
 }
 
-/* only called when we know we can write to a channel, writes as much as
+/* Called to write data out to the server side of a channel (eg a shell or a
+ * program.
+ * Only called when we know we can write to a channel, writes as much as
  * possible */
 static void writechannel(struct Channel* channel) {
 
@@ -257,10 +295,8 @@ static void writechannel(struct Channel* channel) {
 
 	TRACE(("enter writechannel"));
 
-	assert(!channel->sentclosed);
-
-	if (channel->recveof) {
-		TRACE(("leave writechannel: already recveof"));
+	if (channel->recveof || channel->sentclosed) {
+		TRACE(("leave writechannel: already recveof or sentclosed"));
 		return;
 	}
 
@@ -298,9 +334,11 @@ static void writechannel(struct Channel* channel) {
 	TRACE(("leave writechannel"));
 }
 
+/* Set the file descriptors for the main select in session.c
+ * This avoid channels which don't have any window available, are closed, etc*/
 void setchannelfds(fd_set *readfd, fd_set *writefd) {
 	
-	int i;
+	unsigned int i;
 	struct Channel * channel;
 	
 	for (i = 0; i < ses.chansize; i++) {
@@ -325,7 +363,8 @@ void setchannelfds(fd_set *readfd, fd_set *writefd) {
 		}
 		/* stdin */
 		if (channel->infd != -1 && channel->recveof == 0 &&
-				channel->writebuf->pos < channel->writebuf->len) {
+				(channel->writebuf->pos < channel->writebuf->len ||
+				 channel->initconn)) {
 			/* there's space to write more to the program */
 			FD_SET(channel->infd, writefd);
 		}
@@ -349,6 +388,9 @@ void setchannelfds(fd_set *readfd, fd_set *writefd) {
 
 }
 
+/* handle the channel EOF event, by closing the channel filedescriptor. The
+ * channel isn't closed yet, it is left until the incoming (from the program
+ * etc) FD is also EOF */
 void recv_msg_channel_eof() {
 
 	unsigned int chan;
@@ -366,7 +408,8 @@ void recv_msg_channel_eof() {
 	channel->recveof = 1;
 
 	/* we should close the channel */
-	if (channel->type == CHANNEL_ID_X11 || channel->type == CHANNEL_ID_AGENT) {
+	if (channel->type == CHANNEL_ID_X11 || channel->type == CHANNEL_ID_AGENT
+			|| channel->type == CHANNEL_ID_TCPDIRECT) {
 		shutdown(channel->infd, 0);
 	} else {
 		close(channel->infd);
@@ -382,6 +425,7 @@ void recv_msg_channel_eof() {
 }
 
 
+/* Handle channel closure(), respond in kind and close the channels */
 void recv_msg_channel_close() {
 
 	unsigned int chan;
@@ -407,6 +451,8 @@ void recv_msg_channel_close() {
 	TRACE(("leave recv_msg_channel_close"));
 }
 
+/* Remove a channel entry, this is only executed after both sides have sent
+ * channel close */
 static void closechannel(struct Channel * channel) {
 
 	TRACE(("enter closechannel"));
@@ -423,12 +469,22 @@ static void closechannel(struct Channel * channel) {
 		closechansess(channel);
 	}
 
-	ses.channels[channel->index] = NULL;
-	m_free(channel);
+	deletechannel(channel);
 
 	TRACE(("leave closechannel"));
 }
 
+/* Just remove a channel entry, don't try to close descriptors etc */
+static void deletechannel(struct Channel *channel) {
+
+	ses.channels[channel->index] = NULL;
+	m_free(channel);
+	
+}
+
+
+/* Handle channel specific requests, passing off to corresponding handlers
+ * such as chansession or x11fwd */
 void recv_msg_channel_request() {
 
 	unsigned int chan;
@@ -463,7 +519,9 @@ void recv_msg_channel_request() {
 
 }
 
-/* chan is the remote channel, isextended is 0 if it is normal data, 1
+/* Reads data from the server's program/shell/etc, and puts it in a
+ * channel_data packet to send.
+ * chan is the remote channel, isextended is 0 if it is normal data, 1
  * if it is extended data. if it is extended, then the type is in
  * exttype */
 static void send_msg_channel_data(struct Channel *channel, int isextended,
@@ -550,7 +608,8 @@ static void send_msg_channel_data(struct Channel *channel, int isextended,
 }
 
 
-/* when we receive channel data */
+/* when we receive channel data, put it in a buffer for writing to the program/
+ * shell etc */
 void recv_msg_channel_data() {
 
 	unsigned int chan;
@@ -598,6 +657,9 @@ void recv_msg_channel_data() {
 	TRACE(("leave recv_msg_channel_data"));
 }
 
+/* Increment the outgoing data window for a channel - the remote end limits
+ * the amount of data which may be transmitted, this window is decremented
+ * as data is sent, and incremented upon receiving window-adjust messages */
 void recv_msg_channel_window_adjust() {
 
 	unsigned int chan;
@@ -620,6 +682,8 @@ void recv_msg_channel_window_adjust() {
 
 }
 
+/* Increment the incoming data window for a channel, and let the remote
+ * end know */
 static void send_msg_channel_window_adjust(struct Channel* channel, 
 		unsigned int incr) {
 
@@ -633,6 +697,7 @@ static void send_msg_channel_window_adjust(struct Channel* channel,
 	encrypt_packet();
 }
 	
+/* Handle a new channel request, performing any channel-type-specific setup */
 void recv_msg_channel_open() {
 
 	unsigned char* type;
@@ -660,6 +725,10 @@ void recv_msg_channel_open() {
 	}
 	if (strcmp(type, "session") == 0) {
 		typeval = CHANNEL_ID_SESSION;
+#ifndef DISABLE_LOCALTCPFWD
+	} else if (strcmp(type, "direct-tcpip") == 0) {
+		typeval = CHANNEL_ID_TCPDIRECT;
+#endif
 	} else {
 		goto failure;
 	}
@@ -675,6 +744,13 @@ void recv_msg_channel_open() {
 	/* type specific initialisation */
 	if (typeval == CHANNEL_ID_SESSION) {
 		newchansess(channel);
+#ifndef DISABLE_LOCALTCPFWD
+	} else if (typeval == CHANNEL_ID_TCPDIRECT) {
+		if (newtcpdirect(channel) == DROPBEAR_FAILURE) {
+			deletechannel(channel);
+			goto failure;
+		}
+#endif
 	}
 
 	/* success */
@@ -691,6 +767,7 @@ cleanup:
 	TRACE(("leave recv_msg_channel_open"));
 }
 
+/* Send a failure message */
 void send_msg_channel_failure(struct Channel *channel) {
 
 	TRACE(("enter send_msg_channel_failure"));
@@ -703,6 +780,7 @@ void send_msg_channel_failure(struct Channel *channel) {
 	TRACE(("leave send_msg_channel_failure"));
 }
 
+/* Send a success message */
 void send_msg_channel_success(struct Channel *channel) {
 
 	TRACE(("enter send_msg_channel_success"));
@@ -715,6 +793,8 @@ void send_msg_channel_success(struct Channel *channel) {
 	TRACE(("leave send_msg_channel_success"));
 }
 
+/* Send a channel open failure message, with a corresponding reason
+ * code (usually resource shortage or unknown chan type) */
 static void send_msg_channel_open_failure(unsigned int remotechan, 
 		int reason, const unsigned char *text, const unsigned char *lang) {
 
@@ -731,6 +811,8 @@ static void send_msg_channel_open_failure(unsigned int remotechan,
 	TRACE(("leave send_msg_channel_open_failure"));
 }
 
+/* Confirm a channel open, and let the remote end know what number we've
+ * allocated and the receive parameters */
 static void send_msg_channel_open_confirmation(struct Channel* channel,
 		unsigned int recvwindow, 
 		unsigned int recvmaxpacket) {
@@ -783,7 +865,8 @@ int send_msg_channel_open_init(int fd, const char * typestring) {
 	return DROPBEAR_SUCCESS;
 }
 
-/* channel establishment is only required if we have listeners (for x11 etc)*/
+/* Confirmation that our channel open request (for forwardings) was 
+ * successful*/
 void recv_msg_channel_open_confirmation() {
 
 	unsigned int chan;
@@ -804,6 +887,7 @@ void recv_msg_channel_open_confirmation() {
 	TRACE(("leave recv_msg_channel_open_confirmation"));
 }
 
+/* Notification that our channel open request failed */
 void recv_msg_channel_open_failure() {
 
 	unsigned int chan;
