@@ -25,8 +25,7 @@
 #ifndef _CHANNEL_H_
 #define _CHANNEL_H_
 
-#include <sys/types.h>
-#include "options.h"
+#include "includes.h"
 #include "buffer.h"
 
 /* channel->type values */
@@ -34,6 +33,7 @@
 #define CHANNEL_ID_SESSION 1
 #define CHANNEL_ID_X11 2
 #define CHANNEL_ID_AGENT 3
+#define CHANNEL_ID_TCPDIRECT 4
 
 #define SSH_OPEN_ADMINISTRATIVELY_PROHIBITED    1
 #define SSH_OPEN_CONNECT_FAILED                 2
@@ -49,7 +49,7 @@
 
 /* a simpler way to define that we need code for listeners */
 #if !defined(DISABLE_X11FWD) || !defined(DISABLE_AUTHFWD) || \
-	!defined(DISABLE_TCPFWD)
+	!defined(DISABLE_REMOTETCPFWD)
 #define USE_LISTENERS
 #endif
 
@@ -74,6 +74,9 @@ struct Channel {
 	 * or program */
 	int transeof, recveof, erreof; 
 
+	int initconn; /* used for TCP forwarding, whether the channel has been
+					 fully initialised */
+
 };
 	
 void chaninitialise();
