@@ -22,7 +22,7 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE. */
 
-#include "options.h"
+#include "includes.h"
 #include "util.h"
 #include "bignum.h"
 #include "dss.h"
@@ -30,7 +30,12 @@
 #include "ssh.h"
 #include "random.h"
 
-#include "libtomcrypt/mycrypt.h"
+/* Handle DSS (Digital Signature Standard), aka DSA (D.S. Algorithm),
+ * operations, such as key reading, signing, verification. Key generation
+ * is in gendss.c, since it isn't required in the server itself.
+ *
+ * See FIPS186 or the Handbook of Applied Cryptography for details of the
+ * algorithm */
 
 #ifdef DROPBEAR_DSS 
 
@@ -42,13 +47,10 @@ int buf_get_dss_pub_key(buffer* buf, dss_key *key) {
 
 	assert(key != NULL);
 	key->p = m_malloc(sizeof(mp_int));
-	m_mp_init(key->p);
 	key->q = m_malloc(sizeof(mp_int));
-	m_mp_init(key->q);
 	key->g = m_malloc(sizeof(mp_int));
-	m_mp_init(key->g);
 	key->y = m_malloc(sizeof(mp_int));
-	m_mp_init(key->y);
+	m_mp_init_multi(key->p, key->q, key->g, key->y, NULL);
 	key->x = NULL;
 
 	buf_incrpos(buf, 4+SSH_SIGNKEY_DSS_LEN); /* int + "ssh-dss" */
@@ -62,7 +64,7 @@ int buf_get_dss_pub_key(buffer* buf, dss_key *key) {
 	return DROPBEAR_SUCCESS;
 }
 
-/* same as buf_get_dss_pub_key, but reads a private "x" key at the end.
+/* Same as buf_get_dss_pub_key, but reads a private "x" key at the end.
  * Loads a private dss key from a buffer
  * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 int buf_get_dss_priv_key(buffer* buf, dss_key *key) {
@@ -84,7 +86,7 @@ int buf_get_dss_priv_key(buffer* buf, dss_key *key) {
 }
 	
 
-/* clear and free the memory used by a public key */
+/* Clear and free the memory used by a public or private key */
 void dss_key_free(dss_key *key) {
 
 	TRACE(("enter dsa_key_free"));
@@ -116,7 +118,7 @@ void dss_key_free(dss_key *key) {
 	TRACE(("leave dsa_key_free"));
 }
 
-/* put the dss key into the buffer in the required format:
+/* put the dss public key into the buffer in the required format:
  *
  * string	"ssh-dss"
  * mpint	p
@@ -145,7 +147,8 @@ void buf_put_dss_priv_key(buffer* buf, dss_key *key) {
 }
 
 #ifdef DROPBEAR_SIGNKEY_VERIFY
-/* returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
+/* Verify a DSS signature (in buf) made on data by the key given. 
+ * returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 int buf_dss_verify(buffer* buf, dss_key *key, const unsigned char* data,
 		unsigned int len) {
 
@@ -170,10 +173,7 @@ int buf_dss_verify(buffer* buf, dss_key *key, const unsigned char* data,
 	sha1_process(&hs, data, len);
 	sha1_done(&hs, msghash);
 
-	m_mp_init(&val1);
-	m_mp_init(&val2);
-	m_mp_init(&val3);
-	m_mp_init(&val4);
+	m_mp_init_multi(&val1, &val2, &val3, &val4, NULL);
 
 	/* create the signature - s' and r' are the received signatures in buf */
 	/* w = (s')-1 mod q */
@@ -227,16 +227,13 @@ int buf_dss_verify(buffer* buf, dss_key *key, const unsigned char* data,
 	}
 	
 	/* check whether signatures verify */
-	if (mp_cmp(&val2, &val1) == 0) {
+	if (mp_cmp(&val2, &val1) == MP_EQ) {
 		/* good sig */
 		ret = DROPBEAR_SUCCESS;
 	}
 
 out:
-	mp_clear(&val1);
-	mp_clear(&val2);
-	mp_clear(&val3);
-	mp_clear(&val4);
+	mp_clear_multi(&val1, &val2, &val3, &val4, NULL);
 	m_free(string);
 
 	return ret;
@@ -244,10 +241,10 @@ out:
 }
 #endif /* DROPBEAR_SIGNKEY_VERIFY */
 
-/* sign the data presented in len with key, writing the signature contents
+/* Sign the data presented with key, writing the signature contents
  * to the buffer
  *
- * when DSS_PROTOK is #defined:
+ * When DSS_PROTOK is #defined:
  * The alternate k generation method is based on the method used in PuTTY. 
  * In particular to avoid being vulnerable to attacks using flaws in random
  * generation of k, we use the following:
@@ -284,7 +281,8 @@ void buf_put_dss_sign(buffer* buf, dss_key *key, const unsigned char* data,
 	sha1_process(&hs, data, len);
 	sha1_done(&hs, msghash);
 
-	m_mp_init(&dss_k);
+	m_mp_init_multi(&dss_k, &dss_temp1, &dss_temp1, &dss_r, &dss_s,
+			&dss_m, NULL);
 #ifdef DSS_PROTOK	
 	/* hash the privkey */
 	privkeytmp = mptobytes(key->x, &i);
@@ -313,16 +311,11 @@ void buf_put_dss_sign(buffer* buf, dss_key *key, const unsigned char* data,
 		if (mp_read_unsigned_bin(&dss_k, kbuf, SHA1_HASH_SIZE) != MP_OKAY) {
 			dropbear_exit("dss error");
 		}
-	} while (mp_cmp(&dss_k, key->q) >= 0 || mp_cmp_d(&dss_k, 0) <= 0);
+	} while (mp_cmp(&dss_k, key->q) == MP_GT || mp_cmp_d(&dss_k, 0) != MP_GT);
 	m_burn(kbuf, SHA1_HASH_SIZE);
 #endif
 
 	/* now generate the actual signature */
-	m_mp_init(&dss_temp1);
-	m_mp_init(&dss_temp2);
-	m_mp_init(&dss_r);
-	m_mp_init(&dss_s);
-	m_mp_init(&dss_m);
 	bytestomp(&dss_m, msghash, SHA1_HASH_SIZE);
 
 	/* g^k mod p */
@@ -342,22 +335,17 @@ void buf_put_dss_sign(buffer* buf, dss_key *key, const unsigned char* data,
 	if (mp_addmod(&dss_m, &dss_temp1, key->q, &dss_temp2) != MP_OKAY) {
 		dropbear_exit("dss error");
 	}
-	mp_clear(&dss_m);
 	
 	/* (k^-1) mod q */
 	if (mp_invmod(&dss_k, key->q, &dss_temp1) != MP_OKAY) {
 		dropbear_exit("dss error");
 	}
-	mp_clear(&dss_k);
 
 	/* s = (k^-1(SHA1(M) + xr)) mod q */
 	if (mp_mulmod(&dss_temp1, &dss_temp2, key->q, &dss_s) != MP_OKAY) {
 		dropbear_exit("dss error");
 	}
-	mp_clear(&dss_temp1);
-	mp_clear(&dss_temp2);
-	
-	/* create the signature to return */
+
 	buf_putstring(buf, SSH_SIGNKEY_DSS, SSH_SIGNKEY_DSS_LEN);
 	buf_putint(buf, 2*SHA1_HASH_SIZE);
 
@@ -387,6 +375,11 @@ void buf_put_dss_sign(buffer* buf, dss_key *key, const unsigned char* data,
 	mp_clear(&dss_s);
 	buf_incrwritepos(buf, writelen);
 
+	mp_clear_multi(&dss_k, &dss_temp1, &dss_temp1, &dss_r, &dss_s,
+			&dss_m, NULL);
+	
+	/* create the signature to return */
+
 	TRACE(("leave buf_put_dss_sign"));
 }
 
